#include <SoftwareSerial.h>
#include <DFRobotDFPlayerMini.h>
#include <Keypad.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>
#include <SPI.h>
#include <Wire.h>
#include <RTClib.h>
#include <Servo.h>

//KEYPAD

const byte rowsCount = 4;
const byte columsCount = 4;

char keys[rowsCount][columsCount] = {
  {'1', '2', '3', 'A'},
  {'4', '5', '6', 'B'},
  {'7', '8', '9', 'C'},
  {'*', '0', '#', 'D'}
};

const byte rowPins[rowsCount] = { 36, 34, 32, 30 };
const byte columnPins[columsCount] = { 28, 26, 24, 22 };

//DISPLAY
#define ANCHO_PANTALLA 128 // ancho pantalla OLED
#define ALTO_PANTALLA 64 // alto pantalla OLED
Adafruit_SSD1306 display(ANCHO_PANTALLA, ALTO_PANTALLA, &Wire, -1);

// REAL TIME CLOCK
RTC_DS3231 rtc;
String daysOfTheWeek[7] = { "Domingo", "Lunes", "Martes", "Miercoles", "Jueves", "Viernes", "Sabado" };
String monthsNames[12] = { "Enero", "Febrero", "Marzo", "Abril", "Mayo",  "Junio", "Julio", "Agosto", "Septiembre", "Octubre", "Noviembre", "Diciembre" };

// SERVOS
Servo servomotor1, servomotor2, servomotor3;

// DFPlayer Mini
DFRobotDFPlayerMini myDFPlayer;

// VARIABLES
bool codeMode = false;
bool alarm_mode = false;
bool transition_mode = false;
bool displayedCodeMode = false;
char keypadCode[4];
int keypadCodeIndex = 0;
long my_alarms[4] = { 53280, 53340, 53400, 0 };
int alarm_state[4] = { 1, 1, 1, 0 };
int alarm_folder[4] = { 7, 7, 7, 6 };
int alarm_file[4] = { 1, 2, 3, 1 };

Keypad keypad = Keypad(makeKeymap(keys), rowPins, columnPins, rowsCount, columsCount);

String twoDigigtsFormat(String digit) {
  if (digit.length() < 2) {
    return "0" + digit;
  } else {
    return digit;
  }
}
void motorPositioning(Servo servomotor, int degree) {
  servomotor.write(degree);
}
void printDate(DateTime date) {
  int16_t x1, y1;
  uint16_t w, h;
  String text_to_write;

  display.clearDisplay();
  display.setTextColor(SSD1306_WHITE);
  display.setCursor(0, 16);
  display.setTextSize(2);

  text_to_write = String(date.hour()) + ":" + twoDigigtsFormat(String(date.minute()));
  display.getTextBounds(text_to_write, display.getCursorX(), display.getCursorY(), &x1, &y1, &w, &h);
  display.setCursor(floor((ANCHO_PANTALLA - w) / 2), display.getCursorY());
  display.println(text_to_write);

  display.setTextSize(1);
  display.println("");

  text_to_write = String(daysOfTheWeek[date.dayOfTheWeek()]) + ", " + String(date.day()) + "/" + String(date.month()) + "/" + String(date.year());
  display.getTextBounds(text_to_write, display.getCursorX(), display.getCursorY(), &x1, &y1, &w, &h);
  display.setCursor(floor((ANCHO_PANTALLA - w) / 2), display.getCursorY());
  display.println(text_to_write);
  // Enviar a pantalla
  display.display();
}

void interruptHandler() {
  bool status = digitalRead(2);
  if (status == false && codeMode == false && alarm_mode == false) {
    Serial.println("Inicio del modo de inserción de código.");
    codeMode = true;
  }
  if (status == true && codeMode == true && alarm_mode == false) {
    Serial.println("Fin del modo de inserción de código.");
    codeMode = false;
  }
  if (status == false && alarm_mode == true) {
    transition_mode = true;
  }

}

char readKeypad() {
  char key = keypad.getKey();
  return key;
}

void setup() {
  Serial1.begin(9600);
  Serial.begin(9600);

  pinMode(2, INPUT_PULLUP);
  attachInterrupt(digitalPinToInterrupt(2), interruptHandler, CHANGE);

  if (!display.begin(SSD1306_SWITCHCAPVCC, 0x3C)) {
    Serial.println("No se encuentra la pantalla OLED");
  }

  if (!rtc.begin()) {
    Serial.println(F("Módulo RTC no encontrado."));
    while (1);
  }
  if (rtc.lostPower()) {
    // Fijar a fecha y hora de compilacion
    rtc.adjust(DateTime(F(__DATE__), F(__TIME__)));
  }

  if (!myDFPlayer.begin(Serial1)) {
    Serial.println(F("No ha sido posible establecer la conexión con el módulo de musica:"));
  }

  servomotor1.attach(9);
  servomotor2.attach(10);
  servomotor3.attach(11);
  servomotor1.write(180);
  servomotor2.write(180);
  servomotor3.write(180);

  Serial.println(F("Todo listo para el funcionamiento"));
}

void loop() {
  if (transition_mode) {
    myDFPlayer.stop();
    int keyUpAlarm = 0;
    for (int i = 0; i < 4; i++) {
      if (alarm_state[i] == 2) {
        alarm_state[i] = 3;
        keyUpAlarm = i;
        switch (i+1) {
          case 1:
            motorPositioning(servomotor1, 0);
            break;
          case 2:
            motorPositioning(servomotor2, 0);
            break;
          case 3:
            motorPositioning(servomotor3, 0);
            break;
          default:
            break;
        }
      }
    }
    myDFPlayer.playFolder(alarm_folder[keyUpAlarm], alarm_file[keyUpAlarm]);
    delay(2000);
    transition_mode = false;
  }
  if (codeMode) {
    if (!displayedCodeMode) {
      display.clearDisplay();
      display.setTextColor(SSD1306_WHITE);
      display.setCursor(0, 8);
      display.setTextSize(1);
      display.println("Introduzca el comando");
      display.display();
      displayedCodeMode = true;

      display.setTextSize(2);
    }
    char key = keypad.getKey();
    if (key) {
      keypadCode[keypadCodeIndex] = key;

      display.setCursor(keypadCodeIndex * 25 + 18, 36);
      display.print(key);
      display.display();

      keypadCodeIndex += 1;

    }
    if (keypadCodeIndex == 4) {
      keypadCodeIndex = 0;
      displayedCodeMode = false;

      if (keypadCode[0] == '#' && keypadCode[1] == '1' && keypadCode[2] == 'A' && keypadCode[3] == '9') {
        motorPositioning(servomotor1, 0);
        motorPositioning(servomotor2, 0);
        motorPositioning(servomotor3, 0);
      }
      else if (keypadCode[0] == '#' && keypadCode[1] == '1' && keypadCode[2] == 'B' && keypadCode[3] == '9') {
        motorPositioning(servomotor1, 180);
        motorPositioning(servomotor2, 180);
        motorPositioning(servomotor3, 180);
        for (int i = 0; i < 4; i++) {
          if (alarm_state[i] != 0) {
            alarm_state[i] = 1;
          }
        }
      }
      else if (keypadCode[0] == '#' && keypadCode[1] == '1' && keypadCode[2] == 'A') {
        switch (keypadCode[3]) {
          case '1':
            motorPositioning(servomotor1, 0);
            break;
          case '2':
            motorPositioning(servomotor2, 0);
            break;
          case '3':
            motorPositioning(servomotor3, 0);
            break;
          default:
            break;
        }
      }
      else if (keypadCode[0] == '#' && keypadCode[1] == '1' && keypadCode[2] == 'B') {
        switch (keypadCode[3]) {
          case '1':
            motorPositioning(servomotor1, 180);
            break;
          case '2':
            motorPositioning(servomotor2, 180);
            break;
          case '3':
            motorPositioning(servomotor3, 180);
            break;
          default:
            break;
        }
      }
      else if (keypadCode[0] == '#' && keypadCode[1] == '2' && keypadCode[2] == 'A' && keypadCode[3] == '0') {
        myDFPlayer.playFolder(5, 1);
      }
      else if (keypadCode[0] == '#' && keypadCode[1] == '2' && keypadCode[2] == 'B' && keypadCode[3] == '0') {
        myDFPlayer.pause();
      }
      else if (keypadCode[0] == '#' && keypadCode[1] == 'A') {
        int volume_lvl = String(keypadCode[2]).toInt() * 10 + String(keypadCode[3]).toInt();
        myDFPlayer.volume(volume_lvl);
      }
      else if (keypadCode[0] == '#' && keypadCode[1] == 'B') {
        DateTime now = rtc.now();
        int trial_mins = String(keypadCode[2]).toInt() * 10 + String(keypadCode[3]).toInt();
        long alarm_time = String(now.hour()).toInt() * 3600 + (String(now.minute()).toInt() + trial_mins) * 60;
        my_alarms[3] = alarm_time;
        alarm_state[3] = 1;

      }
      else if (keypadCode[0] == '#' && keypadCode[1] == '3' && keypadCode[2] == 'A' && keypadCode[3] == '0') {
        rtc.adjust(DateTime(F(__DATE__), F(__TIME__)));
      }

      else {
        motorPositioning(servomotor1, 0);
        motorPositioning(servomotor2, 0);
        motorPositioning(servomotor3, 0);
      }
    }
  } else {

    Serial.print(alarm_state[0]);
    Serial.print(alarm_state[1]);
    Serial.print(alarm_state[2]);
    Serial.print(alarm_state[3]);
    Serial.println(alarm_mode);

    if (displayedCodeMode) {
      display.clearDisplay();
      display.display();
      displayedCodeMode = false;
    }
    DateTime now = rtc.now();
    long this_time = String(now.hour()).toInt() * 3600 + String(now.minute()).toInt() * 60;
    if (!alarm_mode) {
      printDate(now);
      myDFPlayer.stop();
      
      for (int i = 0; i < 4; i++) {
        if (this_time >= my_alarms[i] && alarm_state[i] == 1) {
          alarm_mode = true;
          myDFPlayer.loopFolder(5);
          display.clearDisplay();
          display.setTextColor(SSD1306_WHITE);
          display.setCursor(0, 0);
          display.setTextSize(1);
          display.println("Toma de medicamentos");
          display.fillRect(0, 28, 86, 22, SSD1306_WHITE);
          display.fillTriangle(86, 16, 128, 40, 86, 64, SSD1306_WHITE);
          display.display();
          alarm_state[i] = 2;
        }
      }
    } else {
      for (int i = 0; i < 4; i++) {
        if (this_time >= my_alarms[i] + 600 && alarm_state[i] == 2) {
          alarm_state[i] = 4;
          alarm_mode = false;
          myDFPlayer.stop();
        }
        if (alarm_state[i] == 3 && transition_mode == false && digitalRead(2) == true) {
          alarm_state[i] = 5;
          myDFPlayer.stop();
          alarm_mode = false;
          transition_mode = false;
          switch (i+1) {
          case 1:
            motorPositioning(servomotor1, 180);
            break;
          case 2:
            motorPositioning(servomotor2, 180);
            break;
          case 3:
            motorPositioning(servomotor3, 180);
            break;
          default:
            break;
        }
        }
        if (this_time >= my_alarms[i] && this_time < my_alarms[i] + 600 && alarm_state[i] == 2){
          alarm_mode = true;
        }
      }
    }

  }
}
