from django.db import models
from .validators import code_format_checker
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC
from cryptography.hazmat.backends import default_backend
import base64

def encodePassword(password):

    backend = default_backend()
    salt = b'#LaCa11eDesiertaLaNocheIdea1-NoPudoEsquivar@'
    kdf = PBKDF2HMAC(
        algorithm=hashes.SHA256(),
        length=32,
        salt=salt,
        iterations=100000,
        backend=backend
    )
    key = password + '#3Patatas100MLDeVinagre6GramosDeSalYPimientaAlGusto@'
    encoded_key = base64.urlsafe_b64encode(kdf.derive(str.encode(key))).decode()
    return encoded_key

# Create your models here.
class AppUser(models.Model):
    id = models.AutoField('Identificador', primary_key=True)
    email = models.EmailField('Correo electrónico', unique=True, max_length=254)
    code = models.CharField('Contraseña', max_length=255, validators=[code_format_checker])
    validated = models.BooleanField('Correo validado', default=False)
    active = models.BooleanField('Cuenta activa', default=False)

    def save(self, *args, **kwargs):
        self.code = encodePassword(self.code)
        super(AppUser, self).save(*args, **kwargs)

    def check_user(self, password):
        encoded_password = encodePassword(password)
        if encoded_password == self.code:
            return True
        else:
            return False

class Administration(models.Model):
    id = models.AutoField('Identificador', primary_key=True)
    name = models.CharField('Nombre', max_length=10, null=False, blank=False)

class Medicine(models.Model):
    id = models.AutoField('Identificador', primary_key=True)
    name = models.CharField('Nombre', max_length=20, null=False, blank=False)
    disp = models.CharField('Disp', max_length=6, unique=True, null=False, blank=False)
    mg = models.FloatField('Miligramos', max_length=5, null=False, blank=False)
    tablets = models.IntegerField('Coprimidos', null=False, blank=False)
    administration = models.ForeignKey(Administration, on_delete=models.SET_NULL, null=True)

    def __str__(self) -> str:
        return self.disp + ' | ' + self.name + ' ' + str(self.mg) + ' mg (' + str(self.tablets) + ' comprimidos)'

class Patient(models.Model):
    id = models.AutoField('Identificador', primary_key=True)
    name = models.CharField('Nombre', max_length=20, null=False, blank=False)
    surname = models.CharField('Apellidos', max_length=30, null=False, blank=False)
    dni = models.CharField('DNI', max_length=9, null=False, blank=False)
    cipa = models.CharField('CIPA', max_length=13, null=False, blank=False)
    birth_date = models.DateField('Fecha de nacimiento', null=False, blank=True)
    phone_number = models.IntegerField('Número de teléfono', null=False, blank=True)
    town = models.CharField('Población', max_length=20, null=False, blank=False)
    province = models.CharField('Provincia', max_length=20, null=False, blank=False)
    direction = models.CharField('Dirección', max_length=60, null=False, blank=False)
    postal_code = models.IntegerField('Código Postal', null=False, blank=False)
    image =  models.CharField('Imagen', max_length=1250 , null=False, blank=False, default='/static/img/patients/default_patient.png')

    def __str__(self) -> str:
        return self.name + ' ' + self.surname + ' | ' + self.dni + ' | ' + self.cipa

class Intake(models.Model):
    id = models.AutoField('Identificador', primary_key=True)
    patient = models.ForeignKey(Patient, on_delete=models.CASCADE, null=False, blank=False)
    moment = models.CharField('Momento', max_length=30, null=False, blank=False, default='Momento personalizado')
    time = models.TimeField('Hora', null=False, blank=False, default='8:00')
    folder = models.IntegerField('Carpeta', null=True, blank=True)
    file = models.IntegerField('Archivo', null=True, blank=True)

    def __str__(self) -> str:
        return self.moment

motor_choices = (
   ('servomotor1', 'Cápsula 1'),
   ('servomotor2', 'Cápsula 2'),
   ('servomotor3', 'Cápsula 3'),
   ('no_servomotor', 'Cápsula no asignada')
)

class Posology(models.Model):
    id = models.AutoField('Identificador', primary_key=True)
    patient = models.ForeignKey(Patient, on_delete=models.CASCADE, null=False, blank=False)
    medicine = models.ForeignKey(Medicine, on_delete=models.CASCADE, null=False, blank=False)
    intake = models.ForeignKey(Intake, on_delete=models.DO_NOTHING, null=False, blank=False)
    chronic = models.BooleanField('Crónica', default=False)
    end_date = models.DateField('Fecha fin', null=True, blank=True)
    amount = models.IntegerField('Cantidad', null=False, blank=False, default=1)
    motor = models.CharField(choices=motor_choices, max_length=128, null=False, blank=False, default='no_servomotor')

class Message(models.Model):
    id = models.AutoField('Identificador', primary_key=True)
    patient = models.ForeignKey(Patient, on_delete=models.CASCADE, null=False, blank=False)
    intake = models.ForeignKey(Intake, on_delete=models.DO_NOTHING, null=False, blank=False)
    state = models.BooleanField('Estado', default=True)