from ast import Return
import datetime
from logging import raiseExceptions
from email.mime.image import MIMEImage
from django.http import HttpResponse, HttpResponseNotFound
from django.template.loader import get_template
from django.core.mail import EmailMultiAlternatives
from django.shortcuts import redirect, render
from django.http.response import JsonResponse
import traceback

from django.core.files.storage import FileSystemStorage
from pastillero_automatico.settings import BASE_DIR

from .models import Administration, AppUser, Intake, Medicine, Message, Patient, Posology
from django.shortcuts import get_object_or_404
from django.forms.models import model_to_dict
from .forms import FormPatient, FormPosology, FormEditPosology, FormIntake, FormEditIntake
import os

from django.core.mail import send_mail
from django.conf import settings
from controlador.ino import writeIno

from rest_framework import authentication, permissions
from rest_framework.views import APIView
authentication_classes = [authentication.TokenAuthentication]
permission_classes = [permissions.IsAdminUser]

def check_session(request):
    if 'user_id' in request.session:
        return True
    else:
        return False

class LoginHandler(APIView):

    def get(self, request):
        if check_session(request):
            return redirect('/cuenta/')
        else:
            return render(request, 'login.html', context={'session': check_session(request),})

    def post(self, request):
        email = request.data['email']
        code = request.data['code']
        if AppUser.objects.filter(email=email).exists():
            account = AppUser.objects.get(email=email)
            if account.validated:
                if account.active:
                    if account.check_user(code):
                        request.session['user_id'] = account.id
                        data = {'validation':'ok',}
                    else:
                        data = {'validation':'not_found',}
                else:
                    data = {'validation':'not_active',}
            else:
                data = {'validation':'not_validated',}
        else:
            data = {'validation':'not_found',}
        return JsonResponse(data, safe=True)

def recover(request):
    if request.method == 'GET':
        return render(request, 'recover.html', context={'session':check_session(request),})
    elif request.method == 'POST':
        email = request.POST['email']
        code = request.POST['code']
        if not AppUser.objects.filter(email=email).exists():
            try:
                AppUser.objects.create(email=email, code=code)
            except:
                data = {'validation':'error',}
            else:
                data = {'validation':'ok',}
        else:
            data = {'validation':'found',}
        return JsonResponse(data, safe=True)

def register(request):
    if request.method == 'GET':
        return render(request, 'register.html', context={'session':check_session(request),})
    elif request.method == 'POST':
        email = request.POST['email']
        code = request.POST['code']
        if not AppUser.objects.filter(email=email).exists():
            try:
                AppUser.objects.create(email=email, code=code)
            except:
                data = {'validation':'error',}
            else:
                data = {'validation':'ok',}
        else:
            data = {'validation':'found',}
        return JsonResponse(data, safe=True)

def account(request):
    if check_session(request):
        if request.method == 'GET':
            return render(request, 'account.html', context={'session': check_session(request),})
    else:
        return redirect('/acceso/')

def messages(request):
    if check_session(request):
        if request.method == 'GET':
            return render(request, 'messages.html', context={'session': check_session(request), 'messages': Message.objects.all()})
    else:
        return redirect('/acceso/')

def logout(request):
    del request.session['user_id']
    return redirect('/acceso/')

def medicines(request):
    if check_session(request):
        if request.method == 'GET':
            context = {
                'session':check_session(request),
                'administration': Administration.objects.all(),
                'medicines': Medicine.objects.all().order_by('name')
            }
            return render(request, 'medicines.html', context=context)
    else:
        return redirect('/acceso/')

class MedicineHandler(APIView):

    def get(self, request):
        if check_session(request):
            context = {
                'session':check_session(request),
                'administration': Administration.objects.all(),
                'medicines': Medicine.objects.all().order_by('name')
            }
            return render(request, 'medicines.html', context=context)
        else:
            return redirect('/acceso/')

    def post(self, request):
        try:
            var_name = request.data['name'].upper()
            var_disp = request.data['disp'].upper()
            var_mg = float(request.data['mg'])
            var_tablets = int(request.data['tablets'])
            id_administration = int(request.data['administration'])
            if var_name and var_disp and var_mg and var_tablets and id_administration:
                Medicine.objects.create(name = var_name, disp = var_disp, mg = var_mg, tablets = var_tablets, administration = Administration.objects.get(id=id_administration))
            else:
                raise Exception('Todos los campos son necesarios')
        except ValueError:
            print(traceback.format_exc())
            data = {'validation':'Revise los formatos'}
        except Exception as error:
            print(traceback.format_exc())
            data = {'validation': error,}
        except:
            print(traceback.format_exc())
            data = {'validation': traceback.format_exc(),}
        else:
            data = {'validation':'ok',}
        return JsonResponse(data, safe=True)

    def put(self, request):
        try:
            var_id = request.data['medicine_id'].upper()
            var_name = request.data['name'].upper()
            var_disp = request.data['disp'].upper()
            var_mg = float(request.data['mg'])
            var_tablets = int(request.data['tablets'])
            id_administration = int(request.data['administration'])
            if Medicine.objects.filter(id = var_id).exists():
                if var_name and var_disp and var_mg and var_tablets and id_administration:
                    Medicine.objects.filter(id = var_id).update(name = var_name, disp = var_disp, mg = var_mg, tablets = var_tablets, administration = Administration.objects.get(id=id_administration))
                else:
                    raise Exception('Todos los campos son necesarios')
            else:
                raise Exception('No se encontró el registro')
        except ValueError:
            print(traceback.format_exc())
            data = {'validation':'Revise los formatos'}
        except Exception as error:
            print(traceback.format_exc())
            data = {'validation': error,}
        except:
            print(traceback.format_exc())
            data = {'validation': traceback.format_exc(),}
        else:
            data = {'validation':'ok',}
        return JsonResponse(data, safe=True)

    def delete(self, request):
        try:
            var_id = request.data['medicine_id'].upper()
            if Medicine.objects.filter(id = var_id).exists():
                Medicine.objects.filter(id = var_id).delete()
            else:
                raise Exception('No se encontró el registro')
        except Exception as error:
            print(traceback.format_exc())
            data = {'validation': error,}
        except:
            print(traceback.format_exc())
            data = {'validation': traceback.format_exc(),}
        else:
            data = {'validation':'ok',}
        return JsonResponse(data, safe=True)

# INICIO BLOQUE PACIENTES ------------------------------------------------------------------

class PatientHandler(APIView):

    def get(self, request, patient_id):
        if check_session(request):
            try:
                initial = Patient.objects.get(id = patient_id)
                instance = get_object_or_404(Patient, id=patient_id)
                form = FormPatient(model_to_dict(initial), instance=instance)
                if form.is_valid():
                    patient = form.cleaned_data
                    data = {'validation':'ok', 'patient': patient, 'image': initial.image,}
                else:
                    print('Invalid form')
                    data = {'validation':'not_ok',}
            except Exception as error:
                print(error)
                traceback.print_exc()
                data = {'validation':'not_ok',}
            return JsonResponse(data, safe=True)
        else:
            return redirect('/acceso/')

    def post(self, request):
        if check_session(request):
            try:
                form = FormPatient(request.data)
                print(form)
                
                if form.is_valid():
                    patient = form.save()

                    Intake.objects.create(patient = patient, moment='Amanecer', time=datetime.time(8, 0))
                    Intake.objects.create(patient = patient, moment='Desayuno', time=datetime.time(8, 30))
                    Intake.objects.create(patient = patient, moment='Almuerzo', time=datetime.time(12, 0))
                    Intake.objects.create(patient = patient, moment='Comida', time=datetime.time(14, 0))
                    Intake.objects.create(patient = patient, moment='Merienda', time=datetime.time(18, 0))
                    Intake.objects.create(patient = patient, moment='Cena', time=datetime.time(21, 0))
                    Intake.objects.create(patient = patient, moment='Anochecer', time=datetime.time(22, 0))
                else:
                    raise ValueError
            except Exception as error:
                traceback.print_exc()
                print(error)
                data = {'validation': 'not_ok',}
            else:
                try:
                    image = request.data['file_image']
                    file_name = "patient_" + str(patient.id) + "." +image.name.split('.')[1]
                except Exception as error:
                    data = {'validation':'ok',}
                else:
                    fs = FileSystemStorage(location = BASE_DIR / "static/img/patients/")
                    fs.save(file_name, image)
                    patient.image = "/static/img/patients/" + file_name
                    patient.save()
                    data = {'validation':'ok',}
            return JsonResponse(data, safe=True)
        else:
            return redirect('/acceso/')

    def put(self, request):
        if check_session(request):
            try:
                instance = get_object_or_404(Patient, id=request.data['id'])
                form = FormPatient(request.data, instance=instance)
                patient = form.save()
            except Exception as error:
                print(error)
                traceback.print_exc()
                data = {'validation': 'not_ok',}
            else:
                try:
                    image = request.data['file_image']
                    file_name = "patient_" + str(patient.id) + "." + image.name.split('.')[1]
                    fs = FileSystemStorage(location = BASE_DIR / "static/img/patients/")
                    if os.path.exists(os.path.join(BASE_DIR / "static/img/patients/", file_name)) and not 'default' in patient.image:
                        os.remove(os.path.join(BASE_DIR / "static/img/patients/", file_name))
                    fs.save(file_name, image)
                    patient.image = "/static/img/patients/" + file_name
                    patient.save()
                    data = {'validation': 'ok',}
                except Exception as error:
                    data = {'validation': 'ok',}
            return JsonResponse(data, safe=True)
        else:
            return redirect('/acceso/')

    def delete(self, request):
        try:
            patient = Patient.objects.get(id = request.data['id'])
            file_name = "patient_" + str(patient.id) + "." + patient.image.split('.')[1]
            if os.path.exists(os.path.join(BASE_DIR / "static/img/patients/", file_name)) and not 'default' in patient.image:
                os.remove(os.path.join(BASE_DIR / "static/img/patients/", file_name))
            patient.delete()
        except Exception as error:
            print(error)
            traceback.format_exc()
            data = {'validation': 'not_ok',}
        else:
            data = {'validation':'ok',}
        return JsonResponse(data, safe=True)

def patients_view(request):
    if check_session(request):
        context = {
            'session':check_session(request),
            'patients': Patient.objects.all().order_by('name')
        }
        return render(request, 'patients.html', context=context)
    else:
        return redirect('/acceso/')

# FINAL BLOQUE PACIENTES -------------------------------------------------------------------

# INICIO BLOQUE PASTILLERO -----------------------------------------------------------------

class PosologyHandler(APIView):
    def get(self, request, posology_id):
        if check_session(request):
            try:
                initial = Posology.objects.get(id=posology_id)
                instance = get_object_or_404(Posology, id=posology_id)
                form = FormEditPosology(model_to_dict(initial), instance=instance)
                if form.is_valid():
                    posology = form.cleaned_data
                    posology['medicine'] = posology['medicine'].id
                    posology['intake'] = posology['intake'].id
                    data = {'validation':'ok', 'posology': posology,}
                else:
                    print('Invalid form')
                    data = {'validation':'not_ok',}
            except Exception as error:
                print(error)
                traceback.print_exc()
                data = {'validation':'not_ok',}
            return JsonResponse(data)
        else:
            return redirect('/acceso/')

    def post(self, request):
        if check_session(request):
            try:
                form = FormPosology(request.data)
                print(form)
                if form.is_valid():
                    posology = form.save()
                    if posology.chronic:
                        posology.end_date = None
                    else:
                        if not posology.end_date:
                            posology.chronic = True
                    if not posology.motor == 'no_servomotor':
                        for single_posology in Posology.objects.filter(patient=posology.patient, intake=posology.intake).exclude(id=posology.id):
                            single_posology.motor = posology.motor
                            single_posology.save()
                        for single_posology in Posology.objects.filter(patient=posology.patient, motor=posology.motor).exclude(intake=posology.intake):
                            single_posology.motor = 'no_servomotor'
                            single_posology.save()
                    else:
                        for single_posology in Posology.objects.filter(patient=posology.patient, intake=posology.intake).exclude(id=posology.id):
                            single_posology.motor = posology.motor
                            single_posology.save()
                    posology.save()
                    data = {'validation':'ok',}
                else:
                    data = {'validation':'not_ok',}
            except Exception as error:
                traceback.print_exc()
                print(error)
                data = {'validation': 'not_ok',}    
            return JsonResponse(data, safe=True)
        else:
            return redirect('/acceso/')

    def put(self, request, posology_id):
        if check_session(request):
            try:
                instance = get_object_or_404(Posology, id=posology_id)
                form = FormEditPosology(request.data, instance=instance)
                posology = form.save()
                if posology.chronic:
                    posology.end_date = None
                else:
                    if not posology.end_date:
                         posology.chronic = True
                if not posology.motor == 'no_servomotor':
                    for single_posology in Posology.objects.filter(patient=posology.patient, intake=posology.intake).exclude(id=posology_id):
                        single_posology.motor = posology.motor
                        single_posology.save()
                    for single_posology in Posology.objects.filter(patient=posology.patient, motor=posology.motor).exclude(intake=posology.intake):
                        single_posology.motor = 'no_servomotor'
                        single_posology.save()
                else:
                    for single_posology in Posology.objects.filter(patient=posology.patient, intake=posology.intake).exclude(id=posology_id):
                        single_posology.motor = posology.motor
                        single_posology.save()
                posology.save()
                data = {'validation': 'ok',}
            except Exception as error:
                print(error)
                traceback.print_exc()
                data = {'validation': 'not_ok',}
            return JsonResponse(data, safe=True)
        else:
            return redirect('/acceso/')

    def delete(self, request, posology_id):
        try:
            posology = Posology.objects.get(id = posology_id)
            posology.delete()
        except Exception as error:
            print(error)
            traceback.format_exc()
            data = {'validation': 'not_ok',}
        else:
            data = {'validation':'ok',}
        return JsonResponse(data, safe=True)

def pillbox_view(request):
    if check_session(request):
        context = {
            'session': check_session(request),
            'patients': Patient.objects.all().order_by('name')
        }
        return render(request, 'pillbox.html', context=context)
    else:
        return redirect('/acceso/')

def personal_pillbox_view(request, patient_id):
    if check_session(request):
        context = {
            'session': check_session(request),
            'posology': Posology.objects.filter(patient_id=patient_id).order_by('intake__time'),
            'medicines': Medicine.objects.all(),
            'intakes': Intake.objects.filter(patient_id=patient_id)
        }
        return render(request, 'personal_pillbox.html', context=context)
    else:
        return redirect('/acceso/')

# FINAL BLOQUE PASTILLERO ------------------------------------------------------------------

# INICIO BLOQUE INGESTA --------------------------------------------------------------------

class IntakeHandler(APIView):
    def get(self, request, intake_id):

        if check_session(request):
            try:
                initial = Intake.objects.get(id = intake_id)
                instance = get_object_or_404(Intake, id = intake_id)
                form = FormEditIntake(model_to_dict(initial), instance=instance)
                if form.is_valid():
                    intake = form.cleaned_data
                    data = {'validation':'ok', 'intake': intake,}
                else:
                    print('Invalid form')
                    data = {'validation':'not_ok',}
            except Exception as error:
                print(error)
                traceback.print_exc()
                data = {'validation':'not_ok',}
            return JsonResponse(data, safe=True)
            
        else:
            return redirect('/acceso/')


    def post(self, request):
        if check_session(request):
            try:
                form = FormIntake(request.data)
                if form.is_valid():
                    form.save()
            except Exception as error:
                traceback.print_exc()
                print(error)
                data = {'validation': 'not_ok',}
            else:
                data = {'validation':'ok',}
            return JsonResponse(data, safe=True)
        else:
            return redirect('/acceso/')

    def put(self, request):
        if check_session(request):
            try:
                instance = get_object_or_404(Intake, id=request.data['id'])
                form = FormEditIntake(request.data, instance=instance)
                if form.is_valid():
                    form.save()
                    data = {'validation': 'ok',}
                else:
                    data = {'validation': 'not_ok',}
            except Exception as error:
                print(error)
                traceback.print_exc()
                data = {'validation': 'not_ok',}
            return JsonResponse(data, safe=True)
        else:
            return redirect('/acceso/')

    def delete(self, request):
        try:
            intake = Intake.objects.get(id = request.data['id'])
            intake.delete()
        except Exception as error:
            print(error)
            traceback.format_exc()
            data = {'validation': 'not_ok',}
        else:
            data = {'validation':'ok',}
        return JsonResponse(data, safe=True)

def intake_view(request, patient_id):
    if check_session(request):
        context = {
            'session': check_session(request),
            'patient': Patient.objects.get(id=patient_id),
            'intakes': Intake.objects.filter(patient_id=patient_id).order_by('time'),
        }
        return render(request, 'intake.html', context=context)
    else:
        return redirect('/acceso/')

# FINAL BLOQUE INGESTA -----------------------------------------------------------------------

def copy_disp(request):
    request.session['medication-disp'] = request.POST['disp']
    data = {'validation':'ok',}
    return JsonResponse(data, safe=True)

def paste_disp(request):
    disp = request.session['medication-disp']
    print(disp)
    data = {'validation':'ok', 'disp': disp}
    return JsonResponse(data, safe=True)

def show(request):
    return render(request, 'init.html', context={'session':check_session(request),})

def send(subject, html, context, email):
    subject = 'Thank you for registering to our site'
    message = ' it  means a world to us '
    email_from = settings.EMAIL_HOST_USER
    recipient_list = ['ivanrzgd@gmail.com',]
    send_mail( subject, message, email_from, recipient_list )
    return True

class MessageHandler(APIView):
    def post(self, request, patient_id, intake_id, state):
        try:
            patient = Patient.objects.get(id=patient_id)
            intake = Intake.objects.get(id=intake_id)
            if state == 0:
                Message.objects.create(patient=patient, intake=intake, state=False)
            else:
                Message.objects.create(patient=patient, intake=intake, state=True)
        except Exception as error:
            print(error)
            traceback.format_exc()
            data = {'validation': 'not_ok',}
        else:
            data = {'validation':'ok',}
        return JsonResponse(data, safe=True)

def generateInoFile(request, patient_id):
    try:
        patient = Patient.objects.get(id=patient_id)
        serv_1 = Posology.objects.filter(patient=patient, motor='servomotor1').first()
        serv_2 = Posology.objects.filter(patient=patient, motor='servomotor2').first()
        serv_3 = Posology.objects.filter(patient=patient, motor='servomotor3').first()
        my_alarms = [0, 0, 0, 0]
        alarm_state = [0, 0, 0, 0]
        alarm_folder = [0, 0, 0, 0]
        alarm_file = [0, 0, 0, 0]
        if serv_1:
            intake_time = serv_1.intake.time.hour * 3600 + serv_1.intake.time.minute * 60
            my_alarms[0] = intake_time
            alarm_state[0] = 1
            if serv_1.intake.folder:
                alarm_folder[0] = serv_1.intake.folder
                alarm_file[0] = serv_1.intake.file
            else:
                alarm_folder[0] = 6
                alarm_file[0] = 1
        if serv_2:
            intake_time = serv_2.intake.time.hour * 3600 + serv_2.intake.time.minute * 60
            my_alarms[1] = intake_time
            alarm_state[1] = 1
            if serv_2.intake.folder:
                alarm_folder[1] = serv_2.intake.folder
                alarm_file[1] = serv_2.intake.file
            else:
                alarm_folder[1] = 6
                alarm_file[1] = 1
        if serv_3:
            intake_time = serv_3.intake.time.hour * 3600 + serv_3.intake.time.minute * 60
            my_alarms[2] = intake_time
            alarm_state[2] = 1
            if serv_3.intake.folder:
                alarm_folder[2] = serv_3.intake.folder
                alarm_file[2] = serv_3.intake.file
            else:
                alarm_folder[2] = 6
                alarm_file[2] = 1
        writeIno(my_alarms, alarm_state, alarm_folder, alarm_file)
        with open("static/files/sketch.txt") as file:
            content = file.read()
            response = HttpResponse(content, content_type='text/plain')
            # response['Content-Length'] = os.path.getsize("static/files/sketch.txt")
            response['Content-Disposition'] = 'attachment; filename=%s' % 'sketch.ino'
        return response
    except Exception as error:
        print(error)
        traceback.format_exc()
        response = HttpResponseNotFound('<h1>Error en el procesado de la petición</h1>')
        

def sendRecoveryMail(request, admin_user):
    
    try:
        htmly = get_template('emails/mailnuevousuario.html')
        url = 'vit.syltec.es/recuperarCuenta/' + admin_user.code + '/'
        top_path = "static/img/emails/top.png"
        top = "top.png"
        bot_path = "static/img/emails/bot.png"
        bot = "bot.png"
        context = {'url': url,}
        subject = 'Bienvenid@ al apartado de empleo de VIT'
        html_content = htmly.render(context)
        text_content = '<a href="vit.syltec.es/recuperarCuenta/' + admin_user.code + '/">Validar usuario</a>'
        msg = EmailMultiAlternatives(subject, text_content, to=[admin_user.email])
        msg.attach_alternative(html_content, "text/html")
        msg.mixed_subtype = 'related'
        with open(top_path, 'rb') as f:
            img = MIMEImage(f.read())
            img.add_header('Content-ID', '<header>')
            img.add_header('Content-Disposition', 'inline', filename=top)
            msg.attach(img)
        with open(bot_path, 'rb') as f:
            img = MIMEImage(f.read())
            img.add_header('Content-ID', '<banner>')
            img.add_header('Content-Disposition', 'inline', filename=bot)
            msg.attach(img)
        msg.send()
    except Exception:
        traceback.print_exc()
        data = {'validation':'not_ok',}
    else:
        data = {'validation':'ok',}

    return JsonResponse(data, safe=False)